@@ -92,14 +92,12 @@ public:
             delete it.value();
         }
 
-        qDeleteAll(m_delayedParseJobs.values());
+        qDeleteAll(m_delayedParseJobs);
     }
 
     // Non-mutex guarded functions, only call with m_mutex acquired.
     void parseDocumentsInternal()
     {
-        kDebug(9505) << "BackgroundParser::parseDocumentsInternal";
-        
         // Create delayed jobs, that is, jobs for documents which have been changed
         // by the user.
         QList<ParseJob*> jobs;
@@ -254,7 +252,7 @@ public:
         int priority() const {
             //Pick the best priority
             int ret = BackgroundParser::WorstPriority;
-            foreach(DocumentParseTarget target, targets)
+            foreach(const DocumentParseTarget &target, targets)
                 if(target.priority < ret)
                     ret = target.priority;
             return ret;
@@ -263,7 +261,7 @@ public:
         TopDUContext::Features features() const {
             //Pick the best features
             TopDUContext::Features ret = (TopDUContext::Features)0;
-            foreach(DocumentParseTarget target, targets)
+            foreach(const DocumentParseTarget &target, targets)
                 ret = (TopDUContext::Features) (ret | target.features);
             return ret;
         }
@@ -271,7 +269,7 @@ public:
         QList<QPointer<QObject> > notifyWhenReady() {
             QList<QPointer<QObject> > ret;
             
-            foreach(DocumentParseTarget target, targets)
+            foreach(const DocumentParseTarget &target, targets)
                 if(target.notifyWhenReady)
                     ret << target.notifyWhenReady;
             
@@ -421,7 +419,7 @@ void BackgroundParser::addDocument(const KUrl& url, TopDUContext::Features featu
 
 void BackgroundParser::addDocumentList(const KUrl::List &urls, TopDUContext::Features features, int priority)
 {
-    foreach (KUrl url, urls)
+    foreach (const KUrl &url, urls)
         addDocument(url, features, priority);
 }
 
@@ -450,7 +448,6 @@ void BackgroundParser::parseComplete(ThreadWeaver::Job* job)
     QMutexLocker lock(&d->m_mutex);
 
     if (ParseJob* parseJob = qobject_cast<ParseJob*>(job)) {
-        kDebug(9505) << "BackgroundParser: parsed" << parseJob->document().str();
 
         emit parseJobFinished(parseJob);
 
@@ -460,8 +457,6 @@ void BackgroundParser::parseComplete(ThreadWeaver::Job* job)
         
         parseJob->setBackgroundParser(0);
 
-		kDebug() << "Queueing job for deletion" << job->metaObject()->className() << "in thread" << QThread::currentThread();
-
 		delete parseJob;
 
         ++d->m_doneParseJobs;
@@ -513,11 +508,9 @@ void BackgroundParser::updateProgressBar()
         d->m_maxParseJobs = 0;
     } else {
         float additionalProgress = 0;
-        for(QMap<KDevelop::ParseJob*, float>::const_iterator it = d->m_jobProgress.begin(); it != d->m_jobProgress.end(); ++it)
+        for(QMap<KDevelop::ParseJob*, float>::const_iterator it = d->m_jobProgress.constBegin(); it != d->m_jobProgress.constEnd(); ++it)
             additionalProgress += *it;
         
-        kDebug() << "progress:" << (additionalProgress + d->m_doneParseJobs)*1000 << "of" << d->m_maxParseJobs*1000;
-        
         emit showProgress(0, d->m_maxParseJobs*1000, (additionalProgress + d->m_doneParseJobs)*1000);
     }
 }
