@@ -37,7 +37,7 @@ DocumentChangeSet::ChangeResult DocumentChangeSet::applyAllChanges() {
     QMap<IndexedString, QString> newTexts;
     QMap<IndexedString, QList<DocumentChangePointer> > filteredSortedChanges;
     
-    foreach(IndexedString file, m_changes.keys())
+    foreach(const IndexedString &file, m_changes.keys())
     {
         CodeRepresentation* repr = createCodeRepresentation(file);
         if(!repr)
@@ -51,7 +51,7 @@ DocumentChangeSet::ChangeResult DocumentChangeSet::applyAllChanges() {
         
         QMultiMap<SimpleCursor, DocumentChangePointer> sortedChanges;
         
-        foreach(DocumentChangePointer change, m_changes[file])
+        foreach(const DocumentChangePointer &change, m_changes[file])
             sortedChanges.insert(change->m_range.end, change);
         
         //Remove duplicates
@@ -65,7 +65,7 @@ DocumentChangeSet::ChangeResult DocumentChangeSet::applyAllChanges() {
                         it = sortedChanges.erase(it);
                         continue;
                     }else{
-                        qDeleteAll(codeRepresentations.values());
+                        qDeleteAll(codeRepresentations);
                         return ChangeResult(QString("Inconsistent change-request at %1").arg(file.str()));
                     }
                 }
@@ -93,7 +93,7 @@ DocumentChangeSet::ChangeResult DocumentChangeSet::applyAllChanges() {
             {
                 textLines[change.m_range.start.line].replace(change.m_range.start.column, change.m_range.end.column-change.m_range.start.column, change.m_newText);
             }else{
-                qDeleteAll(codeRepresentations.values());
+                qDeleteAll(codeRepresentations);
                 return DocumentChangeSet::ChangeResult(QString("Inconsistent change in %1 at %2:%3 -> %4:%5 = %6(encountered %7) -> %8").arg(file.str()).arg(change.m_range.start.line).arg(change.m_range.start.column).arg(change.m_range.end.line).arg(change.m_range.end.column).arg(change.m_oldText).arg(encountered).arg(change.m_newText), sortedChangesList[pos]);
             }
         }
@@ -103,7 +103,7 @@ DocumentChangeSet::ChangeResult DocumentChangeSet::applyAllChanges() {
     
     QMap<IndexedString, QString> oldTexts;
     
-    foreach(IndexedString file, m_changes.keys())
+    foreach(const IndexedString &file, m_changes.keys())
     {
         oldTexts[file] = codeRepresentations[file]->text();
         bool fail = false;
@@ -129,18 +129,18 @@ DocumentChangeSet::ChangeResult DocumentChangeSet::applyAllChanges() {
         
         if(fail) {
             //Fail
-            foreach(IndexedString revertFile, oldTexts.keys())
+            foreach(const IndexedString &revertFile, oldTexts.keys())
                 codeRepresentations[revertFile]->setText(oldTexts[revertFile]);
             
-            qDeleteAll(codeRepresentations.values());
+            qDeleteAll(codeRepresentations);
             return DocumentChangeSet::ChangeResult(QString("Failed to set text on %1").arg(file.str()));
         }
     }
     
     
-    qDeleteAll(codeRepresentations.values());
+    qDeleteAll(codeRepresentations);
     return DocumentChangeSet::ChangeResult(true);
 }
 
 
-}
\ No newline at end of file
+}
