@@ -29,6 +29,8 @@
 #include "indexedstring.h"
 #include "types/abstracttype.h"
 #include "duchainbase.h"
+#include "topducontext.h"
+#include "indexeditems.h"
 
 class QByteArray;
 
@@ -49,134 +51,6 @@ class Declaration;
 class IndexedTopDUContext;
 
 
-///Represents a declaration only by its global indices
-class KDEVPLATFORMLANGUAGE_EXPORT IndexedDeclaration {
-  public:
-    IndexedDeclaration(Declaration* decl = 0);
-    IndexedDeclaration(uint topContext, uint declarationIndex);
-    
-    ///Duchain must be read locked
-    Declaration* declaration() const;
-
-    ///Duchain must be read locked
-    Declaration* data() const {
-      return declaration();
-    }
-
-    bool operator==(const IndexedDeclaration& rhs) const {
-      return m_topContext == rhs.m_topContext && m_declarationIndex == rhs.m_declarationIndex;
-    }
-    
-    uint hash() const {
-      if(isDummy())
-        return 0;
-      return (m_topContext * 53 + m_declarationIndex) * 23;
-    }
-
-    ///@warning The duchain needs to be locked when this is called
-    bool isValid() const {
-      return !isDummy() && declaration() != 0;
-    }
-
-    bool operator<(const IndexedDeclaration& rhs) const {
-      Q_ASSERT(!isDummy());
-      return m_topContext < rhs.m_topContext || (m_topContext == rhs.m_topContext && m_declarationIndex < rhs.m_declarationIndex);
-    }
-
-    ///Index of the Declaration within the top context
-    uint localIndex() const {
-      if(isDummy())
-        return 0;
-      return m_declarationIndex;
-    }
-    
-    uint topContextIndex() const {
-      if(isDummy())
-        return 0;
-      return m_topContext;
-    }
-    
-    IndexedTopDUContext indexedTopContext() const;
-
-    
-    ///The following functions allow storing 2 integers in this object and marking it as a dummy,
-    ///which makes the isValid() function always return false for this object, and use the integers
-    ///for other purposes
-    ///Clears the contained data
-    void setIsDummy(bool dummy) {
-      if(isDummy() == dummy)
-        return;
-      if(dummy)
-        m_topContext = 1 << 31;
-      else
-        m_topContext = 0;
-      m_declarationIndex = 0;
-    }
-    
-    bool isDummy() const {
-      //We use the second highest bit to mark dummies, because the highest is used for the sign bit of stored
-      //integers
-      return (bool)(m_topContext & (1 << 31));
-    }
-    
-    QPair<uint, uint> dummyData() const {
-      Q_ASSERT(isDummy());
-      return qMakePair(m_topContext & (~(1<<31)), m_declarationIndex);
-    }
-    
-    ///Do not call this when this object is valid. The first integer loses one bit of precision.
-    void setDummyData(QPair<uint, uint> data) {
-      Q_ASSERT(isDummy());
-      
-      m_topContext = data.first;
-      m_declarationIndex = data.second;
-      Q_ASSERT(!isDummy());
-      m_topContext |= (1 << 31); //Mark as dummy
-      Q_ASSERT(isDummy());
-      Q_ASSERT(dummyData() == data);
-    }
-
-    
-  private:
-  uint m_topContext;
-  uint m_declarationIndex;
-};
-
-///Represents a declaration only by its index within the top-context
-class KDEVPLATFORMLANGUAGE_EXPORT LocalIndexedDeclaration {
-  public:
-    LocalIndexedDeclaration(Declaration* decl = 0);
-    LocalIndexedDeclaration(uint declarationIndex);
-    //Duchain must be read locked
-
-    Declaration* data(TopDUContext* top) const;
-
-    bool operator==(const LocalIndexedDeclaration& rhs) const {
-      return m_declarationIndex == rhs.m_declarationIndex;
-    }
-    uint hash() const {
-      return m_declarationIndex * 23;
-    }
-
-    bool isValid() const {
-      return m_declarationIndex != 0;
-    }
-
-    bool operator<(const LocalIndexedDeclaration& rhs) const {
-      return m_declarationIndex < rhs.m_declarationIndex;
-    }
-
-    ///Index of the Declaration within the top context
-    uint localIndex() const {
-      return m_declarationIndex;
-    }
-    
-    bool isLoaded(TopDUContext* top) const;
-
-  private:
-  uint m_declarationIndex;
-};
-
 /**
  * \short Represents a single declaration in a definition-use chain.
  *
@@ -256,7 +130,7 @@ public:
    *
    * \param specialization the specialization index (see DeclarationId)
    * \param topContext the top context to search from
-   * \param upDistance upwards distance in the context-structure of the 
+   * \param upDistance upwards distance in the context-structure of the
    *                   given specialization-info. This allows specializing children.
    * */
   virtual Declaration* specialize(uint specialization, const TopDUContext* topContext, int upDistance = 0);
@@ -383,7 +257,7 @@ public:
    *         equals the internal representation. Use this for example to do equality-comparison.
    */
   IndexedIdentifier indexedIdentifier() const;
-  
+
   /**
    * Determine the global qualified identifier of this declaration.
    *
@@ -527,7 +401,7 @@ public:
    * of this declaration to SpecializationStore if it is nonzero.
    */
   virtual void activateSpecialization();
-  
+
   enum {
     Identity = 7
   };
@@ -554,7 +428,7 @@ private:
   virtual Declaration* clonePrivate() const;
 
   void updateCodeModel();
-   
+
   void rebuildDynamicData(DUContext* parent, uint ownIndex);
 
   friend class DUContext;
@@ -571,6 +445,8 @@ inline uint qHash(const IndexedDeclaration& decl) {
 }
 }
 
+Q_DECLARE_METATYPE(KDevelop::IndexedDeclaration)
+
 #endif // DECLARATION_H
 
 // kate: space-indent on; indent-width 2; tab-width 4; replace-tabs on; auto-insert-doxygen on
