@@ -90,7 +90,7 @@ class EnvironmentInformationItem {
     return m_topContext;
   }
 
-  unsigned short int itemSize() const {
+  unsigned int itemSize() const {
     return sizeof(*this) + m_size;
   }
 
@@ -127,7 +127,7 @@ class EnvironmentInformationRequest {
   size_t itemSize() const {
     return sizeof(EnvironmentInformationItem) + DUChainItemSystem::self().dynamicSize(*m_file->d_func());
   }
-
+  
   void createItem(EnvironmentInformationItem* item) const {
     new (item) EnvironmentInformationItem(m_index, DUChainItemSystem::self().dynamicSize(*m_file->d_func()));
     
@@ -248,7 +248,7 @@ class DUChainPrivate
       DUChainPrivate* m_data;
   };
 public:
-  DUChainPrivate() : m_chainsMutex(QMutex::Recursive), instance(0), m_destroyed(false), m_environmentListInfo("Environment Lists"), m_environmentInfo("Environment Information")
+  DUChainPrivate() : m_chainsMutex(QMutex::Recursive), instance(0), m_destroyed(false), m_environmentListInfo("Environment Lists"), m_environmentInfo("Environment Information"), m_cleanupDisabled(false)
   {
     m_chainsByIndex.set_empty_key(0);
     m_chainsByIndex.set_deleted_key(0xffffffff);
@@ -277,18 +277,14 @@ public:
   void clear() {
     QMutexLocker l(&m_chainsMutex);
 
-    //Store all top-contexts to disk
-    for(google::dense_hash_map<uint, TopDUContext*, ItemRepositoryIndexHash>::const_iterator it = m_chainsByIndex.begin(); it != m_chainsByIndex.end(); ++it)
-      (*it).second->m_dynamicData->store();
+    if(!m_cleanupDisabled)
+      doMoreCleanup();
+    
+    DUChainWriteLocker writeLock(DUChain::lock());
 
     for(google::dense_hash_map<uint, TopDUContext*, ItemRepositoryIndexHash>::const_iterator it = m_chainsByIndex.begin(); it != m_chainsByIndex.end(); ++it)
       instance->removeDocumentChain((*it).second);
 
-    {
-        DUChainWriteLocker writeLock(instance->lock());
-        storeAllInformation(true, writeLock);
-    }
-
     m_fileEnvironmentInformations.clear();
 
     Q_ASSERT(m_fileEnvironmentInformations.isEmpty());
@@ -310,6 +306,7 @@ public:
   Definitions m_definitions;
   Uses m_uses;
   QSet<uint> m_loading;
+  bool m_cleanupDisabled;
 
   ///Used to keep alive the top-context that belong to documents loaded in the editor
   QSet<ReferencedTopDUContext> m_openDocumentContexts;
@@ -340,13 +337,16 @@ public:
     Q_ASSERT(info->d_func()->classId);
   }
 
-  ///The item must managed currently
+  ///The item must be managed currently
   void removeEnvironmentInformation(ParsingEnvironmentFilePointer info) {
     
+    info->makeDynamic(); //By doing this, we make sure the data is actually being destroyed in the destructor
+    
     bool removed = (bool)m_fileEnvironmentInformations.remove(info->url(), info);
     uint index = m_environmentInfo.findIndex(info->indexedTopContext().index());
-    if(index)
+    if(index) {
       m_environmentInfo.deleteItem(index);
+    }
     
     Q_ASSERT(index || removed);
   }
@@ -375,7 +375,17 @@ public:
     QMutexLocker l(&m_chainsMutex);
 
     if(m_chainsByIndex.find(index) == m_chainsByIndex.end()) {
-      Q_ASSERT(!m_loading.contains(index)); //When this asserts, a chain was requested that is already being loaded. Must not happen.
+      if(m_loading.contains(index)) {
+        //It's probably being loaded by another thread. So wait until the load is ready
+        while(m_loading.contains(index)) {
+          l.unlock();
+          kDebug() << "waiting for another thread to load index" << index;
+          usleep(50000);
+          l.relock();
+        }
+        loaded.insert(index);
+        return;
+      }
       m_loading.insert(index);
       loaded.insert(index);
       TopDUContext* chain = TopDUContextDynamicData::load(index);
@@ -384,12 +394,14 @@ public:
         chain->setParsingEnvironmentFile(loadInformation(chain->url(), chain->ownIndex()));
 
         l.unlock();
-        //Also load all the imported chains, so they are in the symbol table and the import-structure is built
-        foreach(DUContext::Import import, chain->DUContext::importedParentContexts()) {
-          if(!loaded.contains(import.topContextIndex())) {
-            loadChain(import.topContextIndex(), loaded);
+        if(!chain->usingImportsCache()) {
+          //Eventually also load all the imported chains, so the import-structure is built
+          foreach(const DUContext::Import &import, chain->DUContext::importedParentContexts()) {
+            if(!loaded.contains(import.topContextIndex())) {
+              loadChain(import.topContextIndex(), loaded);
+            }
           }
-        }
+         }
         chain->rebuildDynamicImportStructure();
 
         chain->setInDuChain(true);
@@ -411,7 +423,7 @@ public:
     
     QList<IndexedString> urls = m_fileEnvironmentInformations.keys();
     
-    foreach(IndexedString url, urls) {
+    foreach(const IndexedString &url, urls) {
       
       QMultiMap<IndexedString, ParsingEnvironmentFilePointer>::iterator start = m_fileEnvironmentInformations.lowerBound(url);
       QMultiMap<IndexedString, ParsingEnvironmentFilePointer>::iterator end = m_fileEnvironmentInformations.upperBound(url);
@@ -494,6 +506,8 @@ public:
   ///so the final step where the duchain is permanetly locked is much faster.
   void doMoreCleanup(int retries = 0, bool needLockRepository = true) {
     
+    if(m_cleanupDisabled)
+      return;
     
     //This mutex makes sure that there's never 2 threads at he same time trying to clean up
     static QMutex cleanupMutex(QMutex::Recursive);
@@ -611,16 +625,20 @@ public:
       }
       
       
-      for(QMultiMap<IndexedString, ParsingEnvironmentFilePointer>::iterator it = m_fileEnvironmentInformations.begin(); it != m_fileEnvironmentInformations.end(); ) {
-        ParsingEnvironmentFile* f = (*it).data();
-        Q_ASSERT(f->d_func()->classId);
-        if(f->ref == 1) {
-          //The ParsingEnvironmentFilePointer is only referenced once. This means that it's does not belong to any
-          //loaded top-context, so just remove it to save some memory and processing time.
-          ///@todo use some kind of timeout before removing
-          it = m_fileEnvironmentInformations.erase(it);
-        }else{
-          ++it;
+      if(retries == 0) {
+        //Do this atomically, since we must be sure that _everything_ is already saved
+        for(QMultiMap<IndexedString, ParsingEnvironmentFilePointer>::iterator it = m_fileEnvironmentInformations.begin(); it != m_fileEnvironmentInformations.end(); ) {
+          ParsingEnvironmentFile* f = (*it).data();
+          Q_ASSERT(f->d_func()->classId);
+          if(f->ref == 1) {
+            Q_ASSERT(!f->d_func()->isDynamic()); //It cannot be dynamic, since we have stored before
+            //The ParsingEnvironmentFilePointer is only referenced once. This means that it does not belong to any
+            //loaded top-context, so just remove it to save some memory and processing time.
+            ///@todo use some kind of timeout before removing
+            it = m_fileEnvironmentInformations.erase(it);
+          }else{
+            ++it;
+          }
         }
       }
   
@@ -684,7 +702,6 @@ public:
   }
   
 private:
-  void addRecursiveImports(QSet<TopDUContext*>& contexts, TopDUContext* current);
 
   template<class Entry>
   bool listContains(const Entry entry, const Entry* list, uint listSize) {
@@ -747,14 +764,6 @@ private:
   ItemRepository<EnvironmentInformationItem, EnvironmentInformationRequest> m_environmentInfo;
 };
 
-void DUChainPrivate::addRecursiveImports(QSet<TopDUContext*>& contexts, TopDUContext* current) {
-  if(contexts.contains(current))
-    return;
-  contexts.insert(current);
-  for(RecursiveImports::const_iterator it = current->recursiveImports().constBegin(); it != current->recursiveImports().constEnd(); ++it)
-    addRecursiveImports(contexts, const_cast<TopDUContext*>(it.key()));
-}
-
 K_GLOBAL_STATIC(DUChainPrivate, sdDUChainPrivate)
 
 
@@ -812,6 +821,12 @@ void DUChain::removeDocumentChain( TopDUContext* context )
 {
   QMutexLocker l(&sdDUChainPrivate->m_chainsMutex);
 
+  if(sdDUChainPrivate->m_referenceCounts.contains(context)) {
+  //This happens during shutdown, since everything is unloaded
+  kDebug() << "removed a top-context that was reference-counted:" << context->url().str() << context->ownIndex();
+  sdDUChainPrivate->m_referenceCounts.remove(context);
+  }
+  
   uint index = context->ownIndex();
 
 //   kDebug(9505) << "duchain: removing document" << context->url().str();
@@ -980,7 +995,7 @@ TopDUContext* DUChain::chainForDocument(const IndexedString& document) const
 
   //Eventually load an existing chain from disk
   QList<ParsingEnvironmentFilePointer> list = sdDUChainPrivate->getEnvironmentInformation(document);
-  foreach(ParsingEnvironmentFilePointer file, list) {
+  foreach(const ParsingEnvironmentFilePointer &file, list) {
     if(isInMemory(file->indexedTopContext().index()))
       return file->indexedTopContext().data();
   }
@@ -1135,7 +1150,7 @@ void DUChain::documentAboutToBeDeleted(KTextEditor::Document* doc)
     sc.deconvertDUChain( top );
   }
 
-  foreach(ReferencedTopDUContext top, sdDUChainPrivate->m_openDocumentContexts) {
+  foreach(const ReferencedTopDUContext &top, sdDUChainPrivate->m_openDocumentContexts) {
     if(top->url().str() == doc->url().pathOrUrl())
       sdDUChainPrivate->m_openDocumentContexts.remove(top);
   }
@@ -1199,10 +1214,9 @@ Definitions* DUChain::definitions()
 void DUChain::aboutToQuit()
 {
   sdDUChainPrivate->doMoreCleanup();;
-  DUChainWriteLocker writeLock(lock());
   sdDUChainPrivate->m_openDocumentContexts.clear();
-  sdDUChainPrivate->clear();
   sdDUChainPrivate->m_destroyed = true;
+  sdDUChainPrivate->clear();
 }
 
 uint DUChain::newTopContextIndex() {
@@ -1211,6 +1225,7 @@ uint DUChain::newTopContextIndex() {
 }
 
 void DUChain::refCountUp(TopDUContext* top) {
+  DUChainReadLocker l(lock());
   if(!sdDUChainPrivate->m_referenceCounts.contains(top))
     sdDUChainPrivate->m_referenceCounts.insert(top, 1);
   else
@@ -1218,8 +1233,9 @@ void DUChain::refCountUp(TopDUContext* top) {
 }
 
 void DUChain::refCountDown(TopDUContext* top) {
+  DUChainReadLocker l(lock());
   if(!sdDUChainPrivate->m_referenceCounts.contains(top)) {
-    kWarning() << "tried to decrease reference-count for" << top->url().str() << "but this top-context is not referenced";
+    //kWarning() << "tried to decrease reference-count for" << top->url().str() << "but this top-context is not referenced";
     return;
   }
   --sdDUChainPrivate->m_referenceCounts[top];
@@ -1242,6 +1258,11 @@ void DUChain::updateContextForUrl(const IndexedString& document, TopDUContext::F
   }
 }
 
+void DUChain::disablePersistentStorage() {
+  sdDUChainPrivate->m_cleanupDisabled = true;
+}
+
+
 }
 
 #include "duchain.moc"
