@@ -351,11 +351,17 @@ void DUContextDynamicData::addDeclaration( Declaration * newDeclaration )
   bool inserted = false;
   for (int i = m_context->d_func_dynamic()->m_localDeclarationsSize()-1; i >= 0; --i) {
     Declaration* child = m_context->d_func_dynamic()->m_localDeclarations()[i].data(m_topContext);
-    Q_ASSERT(child);
+    if(!child) {
+      kWarning() << "child declaration number" << i << "of" << m_context->d_func_dynamic()->m_localDeclarationsSize() << "is invalid";
+      continue;
+    }
     if(child == newDeclaration)
       return;
     if (start > child->range().start) {
       insertToArray(m_context->d_func_dynamic()->m_localDeclarationsList(), newDeclaration, i+1);
+      if(!m_context->d_func()->m_localDeclarations()[i+1].data(m_topContext))
+        kFatal() << "Inserted a not addressable declaration";
+      
       inserted = true;
       break;
     }
@@ -456,7 +462,10 @@ bool DUContextDynamicData::removeChildContext( DUContext* context ) {
 void DUContextDynamicData::addImportedChildContext( DUContext * context )
 {
 //   ENSURE_CAN_WRITE
-  Q_ASSERT(!arrayContains(m_context->d_func_dynamic()->m_importersList(), IndexedDUContext(context)));
+  if(arrayContains(m_context->d_func_dynamic()->m_importersList(), IndexedDUContext(context))) {
+    kDebug(9505) << m_context->scopeIdentifier(true).toString() << "importer added multiple times:" << context->scopeIdentifier(true).toString();
+    return;
+  }
 
   m_context->d_func_dynamic()->m_importersList().append(context);
 
@@ -583,6 +592,8 @@ DUContext::~DUContext( )
 
     top->m_dynamicData->clearContextIndex(this);
   }
+  
+  Q_ASSERT(d_func()->isDynamic() == (!top->deleting() || !top->isOnDisk() || top->m_dynamicData->isTemporaryContextIndex(m_dynamicData->m_indexInTopContext)));
 }
 
 QVector< DUContext * > DUContext::childContexts( ) const
@@ -655,6 +666,16 @@ QList<Declaration*> DUContext::findLocalDeclarations( const Identifier& identifi
   return arrayToList(ret);
 }
 
+bool contextIsChildOrEqual(const DUContext* childContext, const DUContext* context) {
+  if(childContext == context)
+    return true;
+  
+  if(childContext->parentContext())
+    return contextIsChildOrEqual(childContext->parentContext(), context);
+  else
+    return false;
+}
+
 void DUContext::findLocalDeclarationsInternal( const Identifier& identifier, const SimpleCursor & position, const AbstractType::Ptr& dataType, DeclarationList& ret, const TopDUContext* /*source*/, SearchFlags flags ) const
 {
   IndexedIdentifier indexedIdentifier(identifier);
@@ -672,7 +693,9 @@ void DUContext::findLocalDeclarationsInternal( const Identifier& identifier, con
             if(alias->aliasedDeclaration().isValid()) {
               declaration = alias->aliasedDeclaration().declaration();
             } else {
+#ifndef Q_CC_MSVC
               kDebug() << "lost aliased declaration";
+#endif
             }
           }
 
@@ -696,39 +719,8 @@ void DUContext::findLocalDeclarationsInternal( const Identifier& identifier, con
 
      Checker checker(flags, dataType, position, type());
 
-     if(!m_dynamicData->m_hasLocalDeclarationsHash) {
+     if(m_dynamicData->m_hasLocalDeclarationsHash) {
        //Use a special hash that contains all declarations visible in this context
-      DUContextDynamicData::VisibleDeclarationIterator it(m_dynamicData);
-      while(it) {
-        Declaration* declaration = *it;
-        if(declaration->indexedIdentifier() == indexedIdentifier) {
-          Declaration* checked = checker.check(declaration);
-          if(checked)
-              ret.append(checked);
-        }
-        ++it;
-      }
-     }else if(d_func()->m_inSymbolTable) {
-       //This context is in the symbol table, use the symbol-table to speed up the search
-       QualifiedIdentifier id(scopeIdentifier(true) + identifier);
-       
-       TopDUContext* top = topContext();
-       
-       uint count;
-       const IndexedDeclaration* declarations;
-       PersistentSymbolTable::self().declarations(id, count, declarations);
-       for(uint a = 0; a < count; ++a) {
-         if(declarations[a].topContextIndex() == top->ownIndex()) {
-           Declaration* decl = LocalIndexedDeclaration(declarations[a].localIndex()).data(top);
-           if(decl) {
-             Declaration* checked = checker.check(decl);
-             if(checked)
-               ret.append(checked);
-           }
-         }
-       }
-     }else{
-       //Iterate through all declarations
       QHash<Identifier, DeclarationPointer>::const_iterator it = m_dynamicData->m_localDeclarationsHash.find(identifier);
       QHash<Identifier, DeclarationPointer>::const_iterator end = m_dynamicData->m_localDeclarationsHash.end();
 
@@ -745,12 +737,44 @@ void DUContext::findLocalDeclarationsInternal( const Identifier& identifier, con
         if(checked)
             ret.append(checked);
       }
-    }
+    }else if(d_func()->m_inSymbolTable && !this->localScopeIdentifier().isEmpty() && !identifier.isEmpty()) {
+       //This context is in the symbol table, use the symbol-table to speed up the search
+       QualifiedIdentifier id(scopeIdentifier(true) + identifier);
+       
+       TopDUContext* top = topContext();
+       
+       uint count;
+       const IndexedDeclaration* declarations;
+       PersistentSymbolTable::self().declarations(id, count, declarations);
+       for(uint a = 0; a < count; ++a) {
+         ///@todo Eventually do efficient iteration-free filtering
+         if(declarations[a].topContextIndex() == top->ownIndex()) {
+           Declaration* decl = LocalIndexedDeclaration(declarations[a].localIndex()).data(top);
+           if(decl && contextIsChildOrEqual(decl->context(), this)) {
+             Declaration* checked = checker.check(decl);
+             if(checked)
+               ret.append(checked);
+           }
+         }
+       }
+     }else {
+       //Iterate through all declarations
+      DUContextDynamicData::VisibleDeclarationIterator it(m_dynamicData);
+      while(it) {
+        Declaration* declaration = *it;
+        if(declaration->indexedIdentifier() == indexedIdentifier) {
+          Declaration* checked = checker.check(declaration);
+          if(checked)
+              ret.append(checked);
+        }
+        ++it;
+      }
+     }
   }
 }
 
-bool DUContext::foundEnough( const DeclarationList& ret ) const {
-  if( !ret.isEmpty() )
+bool DUContext::foundEnough( const DeclarationList& ret, SearchFlags flags ) const {
+  if( !ret.isEmpty() && !(flags & DUContext::NoFiltering))
     return true;
   else
     return false;
@@ -764,7 +788,7 @@ bool DUContext::findDeclarationsInternal( const SearchItem::PtrList & baseIdenti
       if(!baseIdentifiers[a]->isExplicitlyGlobal && baseIdentifiers[a]->next.isEmpty()) //It makes no sense searching locally for qualified identifiers
         findLocalDeclarationsInternal(baseIdentifiers[a]->identifier, position, dataType, ret, source, flags);
 
-    if( foundEnough(ret) )
+    if( foundEnough(ret, flags) )
       return true;
   }
 
@@ -809,7 +833,7 @@ bool DUContext::findDeclarationsInternal( const SearchItem::PtrList & baseIdenti
     }
   }
 
-  if( foundEnough(ret) )
+  if( foundEnough(ret, flags) )
     return true;
 
   ///Step 3: Continue search in parent-context
@@ -1082,7 +1106,7 @@ void DUContext::deleteChildContextsRecursively()
 
   //If we are deleting a context that is already stored to disk, we don't need to load not yet loaded child-contexts.
   //Else we need to, so the declarations are unregistered from symbol-table and from TopDUContextDynamicData in their destructor
-  foreach(LocalIndexedDUContext ctx, children)
+  foreach(const LocalIndexedDUContext &ctx, children)
     if(ctx.isLoaded(top) || !top->deleting() || !top->isOnDisk())
       delete ctx.data(top);
 }
@@ -1137,13 +1161,17 @@ void DUContext::setLocalScopeIdentifier(const QualifiedIdentifier & identifier)
   //DUChain::contextChanged(this, DUChainObserver::Change, DUChainObserver::Identifier);
 }
 
-const QualifiedIdentifier DUContext::localScopeIdentifier() const
+QualifiedIdentifier DUContext::localScopeIdentifier() const
 {
   //ENSURE_CAN_READ Commented out for performance reasons
 
   return d_func()->m_scopeIdentifier;
 }
 
+IndexedQualifiedIdentifier DUContext::indexedLocalScopeIdentifier() const {
+  return d_func()->m_scopeIdentifier;
+}
+
 DUContext::ContextType DUContext::type() const
 {
   //ENSURE_CAN_READ This is disabled, because type() is called very often while searching, and it costs us performance
@@ -1314,7 +1342,7 @@ void DUContext::applyAliases(const SearchItem::PtrList& baseIdentifiers, SearchI
   }
 }
 
-void DUContext::applyUpwardsAliases(SearchItem::PtrList& identifiers, const TopDUContext* source) const {
+void DUContext::applyUpwardsAliases(SearchItem::PtrList& identifiers, const TopDUContext* /*source*/) const {
 
   if(type() == Namespace) {
     QualifiedIdentifier localId = d_func()->m_scopeIdentifier;
