@@ -103,7 +103,7 @@ QString highlightAndEscapeUseText(QString line, uint cutOff, SimpleRange range)
   return Qt::escape(line.left(range.start.column)) + "<span style=\"background-color:yellow\">" + Qt::escape(line.mid(range.start.column, range.end.column - range.start.column)) + "</span>" + Qt::escape(line.mid(range.end.column, line.length() - range.end.column)) ;
 }
 
-OneUseWidget::OneUseWidget(IndexedDeclaration declaration, IndexedString document, SimpleRange range, const CodeRepresentation& code, KTextEditor::SmartRange* smartRange) : m_range(range), m_document(document), m_smartRange(smartRange), m_declaration(declaration) {
+OneUseWidget::OneUseWidget(IndexedDeclaration declaration, IndexedString document, SimpleRange range, const CodeRepresentation& code, KTextEditor::SmartRange* smartRange) : m_range(range), m_smartRange(smartRange), m_declaration(declaration), m_document(document) {
   
   //Make the sizing of this widget independent of the content, because we will adapt the content to the size
   setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
@@ -330,7 +330,7 @@ ContextUsesWidget::ContextUsesWidget(const CodeRepresentation& code, QList<Index
 
       QSet<int> hadIndices;
 
-      foreach(IndexedDeclaration usedDeclaration, usedDeclarations) {
+      foreach(const IndexedDeclaration &usedDeclaration, usedDeclarations) {
         int usedDeclarationIndex = ctx->topContext()->indexForUsedDeclaration(usedDeclaration.data(), false);
         if(hadIndices.contains(usedDeclarationIndex))
           continue;
@@ -349,7 +349,7 @@ ContextUsesWidget::ContextUsesWidget(const CodeRepresentation& code, QList<Index
     QLabel* headerLabel = new QLabel(sizePrefix + "<b>" + i18n("Context") + "</b>" + " <a href='navigateToFunction'>" + Qt::escape(headerText) + "</a>" + sizeSuffix);
     addHeaderItem(headerLabel);
     if(usesCount) {
-      QLabel* usesCountLabel = new QLabel("&nbsp;" + ((usesCount == 1) ? i18n("<i>(1 use)</i>") : i18n("<i>(%1 uses)</i>", usesCount)));
+      QLabel* usesCountLabel = new QLabel("&nbsp;" + i18np("<i>(1 use)</i>", "<i>(%1 uses)</i>", usesCount));
       usesCountLabel->setAlignment(Qt::AlignLeft);
       addHeaderItem(usesCountLabel, Qt::AlignLeft);
     }
@@ -383,7 +383,7 @@ DeclarationsWidget::DeclarationsWidget(const CodeRepresentation& code, QList<Ind
     QLabel* headerLabel = new QLabel(sizePrefix + "<b>" + i18n("Declaration") + "</b>" + sizeSuffix);
     addHeaderItem(headerLabel);
 
-    foreach(IndexedDeclaration decl, declarations)
+    foreach(const IndexedDeclaration &decl, declarations)
       if(decl.data())
         addItem(new OneUseWidget(decl, decl.data()->url(), decl.data()->range(), code, decl.data()->smartRange()));
 
@@ -467,7 +467,7 @@ void TopContextUsesWidget::setExpanded(bool expanded) {
       
       IndexedTopDUContext localTopContext(topContext);
       QList<IndexedDeclaration> localDeclarations;
-      foreach(IndexedDeclaration decl, m_allDeclarations)
+      foreach(const IndexedDeclaration &decl, m_allDeclarations)
         if(decl.indexedTopContext() == localTopContext)
           localDeclarations << decl;
         
