@@ -35,7 +35,7 @@ namespace KDevelop {
 
 uint staticItemRepositoryVersion() {
   //Increase this to reset incompatible item-repositories
-  return 35;
+  return 38;
 }
 
 AbstractItemRepository::~AbstractItemRepository() {
@@ -293,7 +293,7 @@ void ItemRepositoryRegistry::close() {
 ItemRepositoryRegistry::~ItemRepositoryRegistry() {
   close();
   QMutexLocker lock(&m_mutex);
-  foreach(QAtomicInt* counter, m_customCounters.values())
+  foreach(QAtomicInt* counter, m_customCounters)
     delete counter;
 }
 
