@@ -65,15 +65,24 @@ namespace std {
 
 namespace KDevelop
 {
-  
+
+static Utils::BasicSetRepository* recursiveImportRepository() {
+  static Utils::BasicSetRepository recursiveImportRepository("Recursive Imports", false);
+  return &recursiveImportRepository;
+}
+
+Utils::BasicSetRepository* RecursiveImportRepository::repository() {
+  return recursiveImportRepository();
+}
+
 struct DeclarationTopContextExtractor {
-  static IndexedTopDUContext extract(const IndexedDeclaration& decl) {
+  inline static IndexedTopDUContext extract(const IndexedDeclaration& decl) {
     return decl.indexedTopContext();
   }
 };
 
 struct DUContextTopContextExtractor {
-  static IndexedTopDUContext extract(const IndexedDUContext& ctx) {
+  inline static IndexedTopDUContext extract(const IndexedDUContext& ctx) {
     return ctx.indexedTopContext();
   }
 };
@@ -275,11 +284,13 @@ public:
         dynamic_cast<TopDUContext*>(import.context(0))->m_local->m_directImporters.remove(m_ctxt);
   }
   
+  ///@todo Make all this work consistently together with import-caching
+  
   //After loading, should rebuild the links
   void rebuildDynamicImportStructure() {
     //Currently we do not store the whole data in TopDUContextLocalPrivate, so we reconstruct it from what was stored by DUContext.
-    ///@todo sharing
     Q_ASSERT(m_importedContexts.isEmpty());
+    
     FOREACH_FUNCTION(const DUContext::Import& import, m_ctxt->d_func()->m_importedContexts) {
       if(DUChain::self()->isInMemory(import.topContextIndex())) {
         TopDUContext* top = dynamic_cast<TopDUContext*>(import.context(0));
@@ -319,7 +330,7 @@ public:
   
     QSet<QPair<TopDUContext*, const TopDUContext*> > rebuild;
 
-    foreach(DUContext::Import import, m_importedContexts) {
+    foreach(const DUContext::Import &import, m_importedContexts) {
       TopDUContext* top = dynamic_cast<TopDUContext*>(import.context(0));
       if(top) {
         top->m_local->m_directImporters.remove(m_ctxt);
@@ -351,18 +362,18 @@ public:
     if(local)
       m_importedContexts << DUContext::Import(context);
     
-//     if(!m_haveImportStructure)
-//       return;
     foreach(TopDUContext* user, m_dataUsers)
       user->m_local->addImportedContextRecursively(context, temporary, false);
 
 //     context->m_local->needImportStructure();
 
     addImportedContextRecursion(context, context, 1, temporary);
-
-    QHash<const TopDUContext*, QPair<int, const TopDUContext*> > b = context->m_local->m_recursiveImports;
-    for(RecursiveImports::const_iterator it = b.constBegin(); it != b.constEnd(); ++it)
-      addImportedContextRecursion(context, it.key(), (*it).first+1, temporary); //Add contexts that were imported earlier into the given one
+    
+    if(!m_ctxt->usingImportsCache()) {
+      QHash<const TopDUContext*, QPair<int, const TopDUContext*> > b = context->m_local->m_recursiveImports;
+      for(RecursiveImports::const_iterator it = b.constBegin(); it != b.constEnd(); ++it)
+        addImportedContextRecursion(context, it.key(), (*it).first+1, temporary); //Add contexts that were imported earlier into the given one
+    }
   }
 
   //Removes the context from this and all contexts that import this, and manages m_recursiveImports
@@ -374,19 +385,18 @@ public:
     if(local)
       removeFromVector(m_importedContexts, DUContext::Import(context));
     
-//     if(!m_haveImportStructure)
-//       return;
-
     foreach(TopDUContext* user, m_dataUsers)
       user->m_local->removeImportedContextRecursively(context, false);
     
     QSet<QPair<TopDUContext*, const TopDUContext*> > rebuild;
     removeImportedContextRecursion(context, context, 1, rebuild);
 
-    QHash<const TopDUContext*, QPair<int, const TopDUContext*> > b = context->m_local->m_recursiveImports;
-    for(RecursiveImports::const_iterator it = b.constBegin(); it != b.constEnd(); ++it) {
-      if(m_recursiveImports.contains(it.key()) && m_recursiveImports[it.key()].second == context)
-        removeImportedContextRecursion(context, it.key(), it->first+1, rebuild); //Remove all contexts that are imported through the context
+    if(!m_ctxt->usingImportsCache()) {
+      QHash<const TopDUContext*, QPair<int, const TopDUContext*> > b = context->m_local->m_recursiveImports;
+      for(RecursiveImports::const_iterator it = b.constBegin(); it != b.constEnd(); ++it) {
+        if(m_recursiveImports.contains(it.key()) && m_recursiveImports[it.key()].second == context)
+          removeImportedContextRecursion(context, it.key(), it->first+1, rebuild); //Remove all contexts that are imported through the context
+      }
     }
 
     rebuildImportStructureRecursion(rebuild);
@@ -395,8 +405,7 @@ public:
   void removeImportedContextsRecursively(const QList<TopDUContext*>& contexts, bool local) {
     QMutexLocker lock(&importStructureMutex);
 
-//     if(!m_haveImportStructure)
-//       return;
+    
     foreach(TopDUContext* user, m_dataUsers)
       user->m_local->removeImportedContextsRecursively(contexts, false);
 
@@ -409,11 +418,13 @@ public:
         removeFromVector(m_importedContexts, DUContext::Import(context));
       
       removeImportedContextRecursion(context, context, 1, rebuild);
-
-      QHash<const TopDUContext*, QPair<int, const TopDUContext*> > b = context->m_local->m_recursiveImports;
-      for(RecursiveImports::const_iterator it = b.constBegin(); it != b.constEnd(); ++it) {
-        if(m_recursiveImports.contains(it.key()) && m_recursiveImports[it.key()].second == context)
-          removeImportedContextRecursion(context, it.key(), it->first+1, rebuild); //Remove all contexts that are imported through the context
+      
+      if(!m_ctxt->usingImportsCache()) {
+        QHash<const TopDUContext*, QPair<int, const TopDUContext*> > b = context->m_local->m_recursiveImports;
+        for(RecursiveImports::const_iterator it = b.constBegin(); it != b.constEnd(); ++it) {
+          if(m_recursiveImports.contains(it.key()) && m_recursiveImports[it.key()].second == context)
+            removeImportedContextRecursion(context, it.key(), it->first+1, rebuild); //Remove all contexts that are imported through the context
+        }
       }
     }
 
@@ -456,7 +467,7 @@ public:
   //What makes this most complicated is the fact that loops are allowed in the import structure.
   typedef QHash<const TopDUContext*, QPair<int, const TopDUContext*> > RecursiveImports;
   mutable RecursiveImports m_recursiveImports;
-  mutable ConvenientFreeListSet<IndexedTopDUContext, IndexedTopDUContextEmbeddedTreeHandler> m_indexedRecursiveImports;
+  mutable TopDUContext::IndexedRecursiveImports m_indexedRecursiveImports;
   private:
 
 //     void childClosure(QSet<TopDUContext*>& children) {
@@ -472,6 +483,9 @@ public:
 
   void addImportedContextRecursion(const TopDUContext* traceNext, const TopDUContext* imported, int depth, bool temporary = false) {
 
+    if(m_ctxt->usingImportsCache())
+      return;
+    
 //     if(!m_haveImportStructure)
 //       return;
 
@@ -525,6 +539,9 @@ public:
 
   void removeImportedContextRecursion(const TopDUContext* traceNext, const TopDUContext* imported, int distance, QSet<QPair<TopDUContext*, const TopDUContext*> >& rebuild) {
 
+    if(m_ctxt->usingImportsCache())
+       return;
+    
     if(imported == m_ctxt)
       return;
 
@@ -673,54 +690,80 @@ struct TopDUContext::DeclarationChecker {
   DUContext::SearchFlags flags;
 };
 
-ImportTrace TopDUContext::importTrace(const TopDUContext* target) const
-{
-  ImportTrace ret;
-  importTrace(target, ret);
-  return ret;
-}
-
-void TopDUContext::importTrace(const TopDUContext* target, ImportTrace& store) const
-{
-    QMutexLocker lock(&importStructureMutex);
-
-    const TopDUContext* current = this;
-    while(current != target) {
-//       current->d_func()->needImportStructure();
-
-      TopDUContextLocalPrivate::RecursiveImports::const_iterator it = current->m_local->m_recursiveImports.constFind(target);
-
-      if(it == current->m_local->m_recursiveImports.constEnd())
-        return;
-
-      const TopDUContext* nextContext = (*it).second;
-
-      if(nextContext) {
-        store.append(ImportTraceItem(current, current->importPosition(nextContext)));
-
-        current = nextContext;
-      }else{
-        kWarning() << "inconsistent import-structure";
-        return;
-      }
-  }
-}
+// ImportTrace TopDUContext::importTrace(const TopDUContext* target) const
+// {
+//   ImportTrace ret;
+//   importTrace(target, ret);
+//   return ret;
+// }
+// 
+// void TopDUContext::importTrace(const TopDUContext* target, ImportTrace& store) const
+// {
+//     QMutexLocker lock(&importStructureMutex);
+// 
+//     const TopDUContext* current = this;
+//     while(current != target) {
+// //       current->d_func()->needImportStructure();
+// 
+//       TopDUContextLocalPrivate::RecursiveImports::const_iterator it = current->m_local->m_recursiveImports.constFind(target);
+// 
+//       if(it == current->m_local->m_recursiveImports.constEnd())
+//         return;
+// 
+//       const TopDUContext* nextContext = (*it).second;
+// 
+//       if(nextContext) {
+//         store.append(ImportTraceItem(current, current->importPosition(nextContext)));
+// 
+//         current = nextContext;
+//       }else{
+//         kWarning() << "inconsistent import-structure";
+//         return;
+//       }
+//   }
+// }
 
 const TopDUContext::IndexedRecursiveImports& TopDUContext::recursiveImportIndices() const
 {
-  ENSURE_CAN_READ
+//   No lock-check for performance reasons
   QMutexLocker lock(&importStructureMutex);
+  if(!d_func()->m_importsCache.isEmpty())
+    return d_func()->m_importsCache;
   
-  Q_ASSERT(m_local->m_indexedRecursiveImports.contains(ownIndex()));
   return m_local->m_indexedRecursiveImports;
 }
 
+void TopDUContextData::updateImportCacheRecursion(IndexedTopDUContext currentContext, std::set<uint>& visited) {
+  if(visited.find(currentContext.index()) != visited.end())
+    return;
+  Q_ASSERT(currentContext.index()); //The top-context must be in the repository when this is called
+  if(!currentContext.data()) {
+    kDebug() << "importing invalid context";
+    return;
+  }
+  visited.insert(currentContext.index());
+  const TopDUContextData* currentData = currentContext.data()->topContext()->d_func();
+  FOREACH_FUNCTION(const DUContext::Import& import, currentData->m_importedContexts) {
+    IndexedTopDUContext next(import.topContextIndex());
+    if(next.isValid())
+      updateImportCacheRecursion(next, visited);
+  }
+}
 
-RecursiveImports TopDUContext::recursiveImports() const
-{
-  ENSURE_CAN_READ
+
+void TopDUContext::updateImportsCache() {
   QMutexLocker lock(&importStructureMutex);
-  return m_local->m_recursiveImports;
+  std::set<uint> visited;
+  TopDUContextData::updateImportCacheRecursion(this, visited);
+  Q_ASSERT(visited.find(ownIndex()) != visited.end());
+  d_func_dynamic()->m_importsCache = IndexedRecursiveImports(recursiveImportRepository()->createSet(visited));
+  Q_ASSERT(d_func_dynamic()->m_importsCache.contains(IndexedTopDUContext(this)));
+  Q_ASSERT(usingImportsCache());
+  Q_ASSERT(imports(this, SimpleCursor::invalid()));
+}
+
+bool TopDUContext::usingImportsCache() const {
+  return !d_func()->m_importsCache.isEmpty();
 }
 
 SimpleCursor TopDUContext::importPosition(const DUContext* target) const
@@ -802,7 +845,6 @@ TopDUContext::TopDUContext(const IndexedString& url, const SimpleRange& range, P
   
   DUCHAIN_D_DYNAMIC(TopDUContext);
   d->m_features = VisibleDeclarationsAndContexts;
-  d->m_deleting = false;
   d->m_ownIndex = m_local->m_ownIndex;
   m_local->m_file = ParsingEnvironmentFilePointer(file);
   setInSymbolTable(true);
@@ -823,7 +865,7 @@ KSharedPtr<ParsingEnvironmentFile> TopDUContext::parsingEnvironmentFile() const
 TopDUContext::~TopDUContext( )
 {
   if(!m_local->m_sharedDataOwner) {
-    d_func_dynamic()->m_deleting = true;
+    m_dynamicData->m_deleting = true;
     if(!isOnDisk())
       clearUsedDeclarationIndices();
   }
@@ -835,7 +877,7 @@ void TopDUContext::deleteSelf() {
   TopDUContextDynamicData* dynamicData = m_dynamicData;
   
   if(!m_local->m_sharedDataOwner)
-    d_func_dynamic()->m_deleting = true;
+    m_dynamicData->m_deleting = true;
 
   delete this;
   
@@ -893,19 +935,17 @@ struct TopDUContext::FindDeclarationsAcceptor {
     if(!id.isEmpty())
       allDecls = PersistentSymbolTable::self().getDeclarations(id);
     
-    TopDUContext::IndexedRecursiveImports::Iterator imports (top->recursiveImportIndices().iterator());
-
     //This iterator efficiently filters the visible declarations out of all declarations
-    ConvenientEmbeddedSetFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
-                                 IndexedTopDUContextEmbeddedTreeHandler, DeclarationTopContextExtractor> filter;
+    ConvenientEmbeddedSetTreeFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
+                                 IndexedRecursiveImports, DeclarationTopContextExtractor> filter;
     
     //This is used if filterung is disabled
     PersistentSymbolTable::Declarations::Iterator unchecked;
     if(check.flags & DUContext::NoImportsCheck)
       unchecked = allDecls.iterator();
     else
-      filter = ConvenientEmbeddedSetFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
-                                 IndexedTopDUContextEmbeddedTreeHandler, DeclarationTopContextExtractor>(allDecls.iterator(), imports);
+      filter = ConvenientEmbeddedSetTreeFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
+                                 IndexedRecursiveImports, DeclarationTopContextExtractor>(allDecls.iterator(), top->recursiveImportIndices());
                                  
     while(filter || unchecked) {
       
@@ -953,7 +993,7 @@ bool TopDUContext::findDeclarationsInternal(const SearchItem::PtrList& identifie
 
 #ifdef DEBUG_SEARCH
   FOREACH_ARRAY(SearchItem::Ptr idTree, identifiers)
-      foreach(QualifiedIdentifier id, idTree->toList())
+      foreach(const QualifiedIdentifier &id, idTree->toList())
         kDebug() << "findDeclarationsInternal" << id.toString();
 #endif
 
@@ -989,11 +1029,9 @@ void TopDUContext::applyAliases( const AliasChainElement* backPointer, const Sea
     if(!id.isEmpty())
       allDecls = PersistentSymbolTable::self().getDeclarations(id);
     
-    TopDUContext::IndexedRecursiveImports::Iterator imports (recursiveImportIndices().iterator());
-
     //This iterator efficiently filters the visible declarations out of all declarations
-    ConvenientEmbeddedSetFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
-                                 IndexedTopDUContextEmbeddedTreeHandler, DeclarationTopContextExtractor> filter(allDecls.iterator(), imports);
+    ConvenientEmbeddedSetTreeFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
+                                 IndexedRecursiveImports, DeclarationTopContextExtractor> filter(allDecls.iterator(), recursiveImportIndices());
 
     if(filter) {
       DeclarationChecker check(this, position, AbstractType::Ptr(), NoSearchFlags, 0);
@@ -1081,11 +1119,9 @@ void TopDUContext::applyAliases( const AliasChainElement* backPointer, const Sea
     if(!id.isEmpty())
       allDecls = PersistentSymbolTable::self().getDeclarations(id);
     
-    TopDUContext::IndexedRecursiveImports::Iterator imports (recursiveImportIndices().iterator());
-
     //This iterator efficiently filters the visible declarations out of all declarations
-    ConvenientEmbeddedSetFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
-                                 IndexedTopDUContextEmbeddedTreeHandler, DeclarationTopContextExtractor> filter(allDecls.iterator(), imports);
+    ConvenientEmbeddedSetTreeFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
+                                 IndexedRecursiveImports, DeclarationTopContextExtractor> filter(allDecls.iterator(), recursiveImportIndices());
     
       
     if(filter) {
@@ -1160,19 +1196,17 @@ struct TopDUContext::FindContextsAcceptor {
     QualifiedIdentifier id = element.qualifiedIdentifier();
     allDecls = PersistentSymbolTable::self().getContexts(id);
     
-    TopDUContext::IndexedRecursiveImports::Iterator imports (top->recursiveImportIndices().iterator());
-
     //This iterator efficiently filters the visible declarations out of all declarations
-    ConvenientEmbeddedSetFilterIterator<IndexedDUContext, IndexedDUContextHandler, IndexedTopDUContext, 
-                                 IndexedTopDUContextEmbeddedTreeHandler, DUContextTopContextExtractor> filter;
+    ConvenientEmbeddedSetTreeFilterIterator<IndexedDUContext, IndexedDUContextHandler, IndexedTopDUContext, 
+                                 IndexedRecursiveImports, DUContextTopContextExtractor> filter;
     
     //This is used if filterung is disabled
     PersistentSymbolTable::Contexts::Iterator unchecked;
     if(check.flags & DUContext::NoImportsCheck)
       unchecked = allDecls.iterator();
     else
-      filter = ConvenientEmbeddedSetFilterIterator<IndexedDUContext, IndexedDUContextHandler, IndexedTopDUContext, 
-                                 IndexedTopDUContextEmbeddedTreeHandler, DUContextTopContextExtractor>(allDecls.iterator(), imports);
+      filter = ConvenientEmbeddedSetTreeFilterIterator<IndexedDUContext, IndexedDUContextHandler, IndexedTopDUContext, 
+                                 IndexedRecursiveImports, DUContextTopContextExtractor>(allDecls.iterator(), top->recursiveImportIndices());
 
     while(filter || unchecked) {
       
@@ -1225,7 +1259,8 @@ TopDUContext * TopDUContext::topContext() const
 
 bool TopDUContext::deleting() const
 {
-  return d_func()->m_deleting;
+  ///@todo remove d_func()->m_deleting, not used any more
+  return m_dynamicData->m_deleting;
 }
 
 QList<ProblemPointer> TopDUContext::problems() const
@@ -1264,17 +1299,11 @@ QList<DUContext*> TopDUContext::loadedImporters() const
 QVector<DUContext::Import> TopDUContext::importedParentContexts() const
 {
   ENSURE_CAN_READ
-
-  if(m_local->m_sharedDataOwner)
-    return m_local->m_importedContexts + m_local->m_sharedDataOwner->m_local->m_importedContexts;
-  else
-    return m_local->m_importedContexts;
+  return DUContext::importedParentContexts();
 }
 
 bool TopDUContext::imports(const DUContext * origin, const SimpleCursor& position) const
 {
-  ENSURE_CAN_READ
-
   return importsPrivate(origin, position);
 }
 
@@ -1282,27 +1311,33 @@ bool TopDUContext::importsPrivate(const DUContext * origin, const SimpleCursor&
 {
   Q_UNUSED(position);
 
-  if( dynamic_cast<const TopDUContext*>(origin) ) {
+  if( const TopDUContext* top = dynamic_cast<const TopDUContext*>(origin) ) {
     QMutexLocker lock(&importStructureMutex);
-//     d_func()->needImportStructure();
-    return m_local->m_recursiveImports.contains(static_cast<const TopDUContext*>(origin));
+    bool ret = recursiveImportIndices().contains(IndexedTopDUContext(const_cast<TopDUContext*>(top)));
+    if(top == this)
+      Q_ASSERT(ret);
+    return ret;
   } else {
     return DUContext::imports(origin, position);
   }
  }
 
 void TopDUContext::clearImportedParentContexts() {
+  if(usingImportsCache()) {
+    d_func_dynamic()->m_importsCache = IndexedRecursiveImports();
+    d_func_dynamic()->m_importsCache.insert(IndexedTopDUContext(this));
+  }
   m_local->clearImportedContextsRecursively();
   
   if(!m_local->m_sharedDataOwner)
     DUContext::clearImportedParentContexts();
   else {
-    foreach (Import parent, m_local->m_importedContexts)
+    foreach (const Import &parent, m_local->m_importedContexts)
       if( parent.context(0) )
         removeImportedParentContext(parent.context(0));
     Q_ASSERT(m_local->m_importedContexts.isEmpty());
   }
-  
+  Q_ASSERT(imports(this, SimpleCursor::invalid()));
 }
 
 void TopDUContext::addImportedParentContext(DUContext* context, const SimpleCursor& position, bool anonymous, bool temporary) {
@@ -1324,7 +1359,7 @@ void TopDUContext::removeImportedParentContext(DUContext* context) {
 void TopDUContext::addImportedParentContexts(const QList<QPair<TopDUContext*, SimpleCursor> >& contexts, bool temporary) {
   typedef QPair<TopDUContext*, SimpleCursor> Pair;
 
-  foreach(Pair pair, contexts)
+  foreach(const Pair &pair, contexts)
     addImportedParentContext(pair.first, pair.second, false, temporary);
 }
 
