@@ -22,7 +22,8 @@
 #include <QtCore/QMutex>
 
 #include "ducontext.h"
-#include <util/convenientfreelist.h>
+#include <language/util/setrepository.h>
+#include "indexeditems.h"
 
 template< class T >
 class KSharedPtr;
@@ -40,6 +41,10 @@ namespace KDevelop
   class DeclarationChecker;
   class TopDUContext;
 
+  struct KDEVPLATFORMLANGUAGE_EXPORT RecursiveImportRepository {
+    static Utils::BasicSetRepository* repository();
+  };
+  
   ///Maps an imported top-context to a pair:
   ///1. The distance to the top-context, and 2. The next step towards the top-context
   ///in the chain.
@@ -59,27 +64,27 @@ class KDEVPLATFORMLANGUAGE_EXPORT ReferencedTopDUContext {
     
     ReferencedTopDUContext& operator=(const ReferencedTopDUContext& rhs);
     
-    TopDUContext* data() const {
+    inline TopDUContext* data() const {
       return m_topContext;
     }
     
-    operator TopDUContext*() const {
+    inline operator TopDUContext*() const {
       return m_topContext;
     }
     
-    bool operator==(const ReferencedTopDUContext& rhs) const {
+    inline bool operator==(const ReferencedTopDUContext& rhs) const {
       return m_topContext == rhs.m_topContext;
     }
     
-    bool operator!=(const ReferencedTopDUContext& rhs) const {
+    inline bool operator!=(const ReferencedTopDUContext& rhs) const {
       return m_topContext != rhs.m_topContext;
     }
     
-    TopDUContext* operator->() const {
+    inline TopDUContext* operator->() const {
       return m_topContext;
     }
     
-    uint hash() const {
+    inline uint hash() const {
       return (uint)(((quint64)m_topContext) * 37);
     }
     
@@ -87,109 +92,6 @@ class KDEVPLATFORMLANGUAGE_EXPORT ReferencedTopDUContext {
   TopDUContext* m_topContext;
 };
 
-///Allows simple indirect access to top-contexts with on-demand loading
-class KDEVPLATFORMLANGUAGE_EXPORT IndexedTopDUContext {
-  public:
-    IndexedTopDUContext(uint index) : m_index(index) {
-      if(!index)
-        setIsDummy(true);
-    }
-    IndexedTopDUContext(TopDUContext* context = 0);
-    
-    enum {
-      DummyMask = 1<<31
-    };
-    
-    ///Returns the top-context represented by this indexed top-context. If it wasn't loaded yet, it is loaded.
-    ///The duchain must be read-locked when this is called!
-    ///To prevent it from being unloaded, store it into a ReferencedTopDUContext before releasing the duchain lock.
-    TopDUContext* data() const;
-    
-    ///Returns whether the top-context is currently loaded. If not, it will be loaded when you call data().
-    bool isLoaded() const;
-    
-    bool operator==(const IndexedTopDUContext& rhs) const {
-      return m_index == rhs.m_index;
-    }
-    bool operator!=(const IndexedTopDUContext& rhs) const {
-      return m_index != rhs.m_index;
-    }
-    
-    bool operator<(const IndexedTopDUContext& rhs) const {
-      return m_index < rhs.m_index;
-    }
-    
-    uint index() const {
-      if(isDummy())
-        return 0;
-      else
-        return m_index;
-    }
-    
-    bool isDummy() const {
-      return m_index & DummyMask;
-    }
-    
-    void setIsDummy(bool isDummy) {
-      if(isDummy)
-        m_index |= DummyMask;
-      else
-        m_index &= ~((uint)DummyMask);
-    }
-    
-    ///Allows giving this IndexedTopDUContext some data while logically keeping it invalid.
-    ///It will still return zero on index(), data(), etc.
-    ///@param first The highest of this value bit will be removed
-    void setDummyData(ushort first, ushort second) {
-      Q_ASSERT(isDummy());
-      m_index = ((((uint)first)<<16) + second) | DummyMask;
-    }
-
-    ///The data previously set through setDummyData(). Initially 0.
-    QPair<ushort, ushort> dummyData() const {
-      uint withoutMask = m_index & (~((uint)DummyMask));
-      return qMakePair((ushort)(withoutMask >> 16), (ushort)withoutMask);
-    }
-    
-    IndexedString url() const;
-  private:
-  uint m_index;
-  friend class IndexedTopDUContextEmbeddedTreeHandler;
-};
-
-class IndexedTopDUContextEmbeddedTreeHandler {
-    public:
-    static int leftChild(const IndexedTopDUContext& m_data) {
-        return int(m_data.dummyData().first)-1;
-    }
-    static void setLeftChild(IndexedTopDUContext& m_data, int child) {
-        m_data.setDummyData((ushort)(child+1), m_data.dummyData().second);
-    }
-    static int rightChild(const IndexedTopDUContext& m_data) {
-        return int(m_data.dummyData().second)-1;
-    }
-    static void setRightChild(IndexedTopDUContext& m_data, int child) {
-        m_data.setDummyData(m_data.dummyData().first, (ushort)(child+1));
-    }
-    static void createFreeItem(IndexedTopDUContext& data) {
-        data = IndexedTopDUContext();
-        data.setIsDummy(true);
-        data.setDummyData(0u, 0u); //Since we substract 1, this equals children -1, -1
-    }
-    //Copies this item into the given one
-    static void copyTo(const IndexedTopDUContext& m_data, IndexedTopDUContext& data) {
-        data = m_data;
-    }
-    static bool isFree(const IndexedTopDUContext& m_data) {
-        return m_data.isDummy();
-    }
-
-    static bool equals(const IndexedTopDUContext& m_data, const IndexedTopDUContext& rhs) {
-      return m_data == rhs;
-    }
-};
-
-
 /**
  * The top context in a definition-use chain for one source file.
  *
@@ -271,8 +173,11 @@ public:
   void clearProblems();
 
   /**
-   * Determine if this chain imports another chain.
-   *
+   * Determine if this chain imports another chain recursively.
+   * 
+   * This uses the imports-cache for speedup if it is available, thus it is not necessarily 100% correct
+   * if the cache is not up-to-date.
+   * 
    * \note you must be holding a read but not a write chain lock when you access this function.
    */
   virtual bool imports(const DUContext* origin, const SimpleCursor& position) const;
@@ -282,10 +187,10 @@ public:
    * The positions in the returned trace may be invalid.
    * This is more efficient then the version below, because the trace doesn't need to be copied
    * */
-  void importTrace(const TopDUContext* target, ImportTrace& store) const;
+//   void importTrace(const TopDUContext* target, ImportTrace& store) const;
 
   ///More convenient version of the above
-  ImportTrace importTrace(const TopDUContext* target) const;
+//   ImportTrace importTrace(const TopDUContext* target) const;
 
   enum {
     Identity = 4
@@ -369,16 +274,7 @@ public:
   ///When this top-context does not own its private data, only the local imports of this context are removed, not those from the shared data.
   virtual void clearImportedParentContexts();
   
-  typedef ConvenientFreeListSet<IndexedTopDUContext, IndexedTopDUContextEmbeddedTreeHandler> IndexedRecursiveImports;
-  
-  ///@todo Create a cache of recursive imports that is stored to disk, so we don't need to load all imports when loading a file
-  ///A cached set of all top-contexts that are recursively imported into this one. It is updated when updateRecursiveImports() is called,
-  ///when it is first used after an import was removed/added.
-//   RecursiveImportsIterator recursiveImportsCache();
-  
-  ///Updates the recursive imports cache. This needs to be called whenever the top-context or one of its imports has been updated.
-  ///The cache is automatically cleared when an import is added/removed
-//   void updateRecursiveImports();
+  typedef Utils::StorableSet<IndexedTopDUContext, IndexedTopDUContextIndexConversion, RecursiveImportRepository, true> IndexedRecursiveImports;
   
   virtual QVector<Import> importedParentContexts() const;
   
@@ -407,7 +303,26 @@ public:
     CacheData* d;
   };
   
+  ///Returns the set of all recursively imported top-contexts. If import-caching is used, this returns the cached set.
+  ///The list also contains this context itself. This set is used to determine declaration-visibility from within this top-context.
+  const IndexedRecursiveImports& recursiveImportIndices() const;
+
+  /**
+   * Updates the cache of recursive imports. When you call this, from that moment on the set returned by recursiveImportIndices() is fixed, until
+   * you call it again to update them. If your language has a very complex often-changing import-structure, 
+   * like for example in the case of C++, it is recommended to call this during while parsing, instead of using
+   * the expensive builtin implicit mechanism.
+   * Note that if you use caching, you _must_ call this before you see any visibility-effect after adding imports.
+   *
+   * Using import-caching has another big advantage: A top-context can be loaded without loading all its imports.
+   * 
+   * Note: This is relatively expensive since it requires loading all imported contexts.
+   * 
+   * When this is called, the top-context must already be registered in the duchain.
+   */
+  void updateImportsCache();
   
+  bool usingImportsCache() const;
 
   virtual bool findDeclarationsInternal(const SearchItem::PtrList& identifiers, const SimpleCursor& position, const AbstractType::Ptr& dataType, DeclarationList& ret, const TopDUContext* source, SearchFlags flags) const;
 protected:
@@ -426,13 +341,6 @@ protected:
 protected:
 
   virtual ~TopDUContext();
-private:
-  ///Returns a set of all recursively imported top-contexts. Each imported top-context is mapped to the distance, and the direct
-  ///import through which the top-context is imported.
-  RecursiveImports recursiveImports() const;
-  ///Returns the indices of all recursively imported top-contexts
-  ///The list also contains this context itself
-  const IndexedRecursiveImports& recursiveImportIndices() const;
   
   void rebuildDynamicData(DUContext* parent, uint ownIndex);
   //Must be called after all imported top-contexts were loaded into the du-chain
@@ -453,6 +361,7 @@ private:
   //Most of these classes need access to m_dynamicData
   friend class DUChain;
   friend class DUChainPrivate;
+  friend class TopDUContextData;
   friend class TopDUContextLocalPrivate;
   friend class TopDUContextDynamicData;
   friend class Declaration;
