@@ -59,7 +59,7 @@ void saveDUChainItem(QList<ArrayWithPosition>& data, DUChainBase& item, uint& to
   item.setData(&target);
 }
 
-TopDUContextDynamicData::TopDUContextDynamicData(TopDUContext* topContext) : m_topContext(topContext), m_fastContexts(0), m_fastContextsSize(0), m_fastDeclarations(0), m_fastDeclarationsSize(0), m_onDisk(false), m_dataLoaded(true) {
+TopDUContextDynamicData::TopDUContextDynamicData(TopDUContext* topContext) : m_topContext(topContext), m_fastContexts(0), m_fastContextsSize(0), m_fastDeclarations(0), m_fastDeclarationsSize(0), m_onDisk(false), m_dataLoaded(true), m_deleting(false) {
 }
 
 TopDUContextDynamicData::~TopDUContextDynamicData() {
@@ -206,12 +206,16 @@ TopDUContext* TopDUContextDynamicData::load(uint topContextIndex) {
 
     TopDUContextDynamicData& target(*ret->m_dynamicData);
     
+//     kDebug() << "loaded" << ret->url().str() << ret->ownIndex() << "import-count:" << ret->importedParentContexts().size() << ret->d_func()->m_importedContextsSize();
+    
     target.m_data.clear();
     target.m_dataLoaded = false;
     target.m_onDisk = true;
     ret->rebuildDynamicData(0, topContextIndex);
     target.m_topContextData.append(qMakePair(topContextData, (uint)0));
     
+//     kDebug() << "loaded" << ret->url().str() << ret->ownIndex() << "import-count:" << ret->importedParentContexts().size() << ret->d_func()->m_importedContextsSize();
+    
     return ret;
   }else{
     kWarning() << "Cannot open top-context for reading:" << fileName;
@@ -224,6 +228,8 @@ bool TopDUContextDynamicData::isOnDisk() const {
 }
 
 void TopDUContextDynamicData::store() {
+//   kDebug() << "storing" << m_topContext->url().str() << m_topContext->ownIndex() << "import-count:" << m_topContext->importedParentContexts().size();
+  
   if(m_onDisk) {
     //Check if something has changed. If nothing has changed, don't store to disk.
     bool someThingChanged = false;
@@ -278,7 +284,7 @@ void TopDUContextDynamicData::store() {
     m_topContextData.clear();
 
     uint newDataSize = 0;
-    foreach(ArrayWithPosition array, oldDatas)
+    foreach(const ArrayWithPosition &array, oldDatas)
         newDataSize += array.second;
     
     //We always put 1 byte to the front, so we don't have zero data-offsets, since those are used for "invalid".
@@ -352,6 +358,7 @@ void TopDUContextDynamicData::store() {
   }else{
     kWarning() << "Cannot open top-context for writing";
   }
+//   kDebug() << "stored" << m_topContext->url().str() << m_topContext->ownIndex() << "import-count:" << m_topContext->importedParentContexts().size();
 }
 
 uint TopDUContextDynamicData::allocateDeclarationIndex(Declaration* decl, bool temporary) {
@@ -400,8 +407,10 @@ Declaration* TopDUContextDynamicData::getDeclarationForIndex(uint index) const {
     loadData();
   
   if(index < (0x0fffffff/2)) {
-    if(index == 0 || index > uint(m_fastDeclarationsSize))
+    if(index == 0 || index > uint(m_fastDeclarationsSize)) {
+      kWarning() << "index out of bounds:" << index << "count:" << m_fastDeclarationsSize;
       return 0;
+    }
     else {
       uint realIndex = index-1;
       if(!m_fastDeclarations[realIndex] && realIndex < (uint)m_declarationDataOffsets.size() && m_declarationDataOffsets[realIndex].dataOffset) {
@@ -473,6 +482,14 @@ uint TopDUContextDynamicData::allocateContextIndex(DUContext* decl, bool tempora
   }
 }
 
+bool TopDUContextDynamicData::isTemporaryContextIndex(uint index) const {
+  return !(index < (0x0fffffff/2));
+}
+
+bool TopDUContextDynamicData::isTemporaryDeclarationIndex(uint index) const {
+  return !(index < (0x0fffffff/2));
+}
+
 DUContext* TopDUContextDynamicData::getContextForIndex(uint index) const {
   
   if(!m_dataLoaded)
