@@ -212,49 +212,16 @@ QString printSplitTree(SplitTreeNode* tree, QString prefix = "") {
     return ret;
 }
 
-struct SetNodeData {
-  //Rule: start <= end
-  Index start, end; //This set-node bounds all indices starting at start until end, not including end.
-
-  //Child nodes
-  //Rule: left->start == start, right->end == end
-  //Rule: (left != 0 && right != 0) || (left == 0 && right == 0)
-  //Ptr left, right;
-  uint leftNode, rightNode;
-  uint m_hash;
-  uint m_refCount;
-  
-  SetNodeData() : start(1), end(1), leftNode(0), rightNode(0), m_hash(0), m_refCount(0) {
-  }
-  
-  uint hash() const {
-    return m_hash;
-  }
-  
-  short unsigned int itemSize() const {
-    return sizeof(SetNodeData);
-  }
-  
-  inline bool contiguous() const {
-    return !leftNode;
-  }
-  
-  inline bool hasSlaves() const {
-    return (bool)leftNode;
-  }
-  
-  //Must always be called when an attribute was changed!
-  void updateHash(const SetNodeData* left, const SetNodeData* right) {
+void SetNodeData::updateHash(const SetNodeData* left, const SetNodeData* right) {
     m_hash = 0;
     if(left && right) {
-      m_hash = left->m_hash + right->m_hash;
+    m_hash = left->m_hash + right->m_hash;
     } else
-      m_hash = hashFromRange(start, end);
-  }
-};
+    m_hash = hashFromRange(start, end);
+}
 
 class SetNodeDataRequest;
-typedef KDevelop::ItemRepository<SetNodeData, SetNodeDataRequest, KDevelop::NoDynamicData, false> SetDataRepository;
+typedef KDevelop::ItemRepository<SetNodeData, SetNodeDataRequest, KDevelop::NoDynamicData, false, sizeof(SetNodeData)> SetDataRepository;
 
 struct SetRepositoryAlgorithms {
   SetRepositoryAlgorithms(SetDataRepository& _repository) : repository(_repository) {
@@ -970,11 +937,15 @@ bool SetRepositoryAlgorithms::set_contains(const SetNodeData* node, Index index)
 
     if(node->contiguous())
       return true;
+    
+    const SetNodeData* leftNode = nodeFromIndex(node->leftNode);
 
-    if(index < getLeftNode(node)->end)
-      node = nodeFromIndex(node->leftNode);
-    else
-      node = nodeFromIndex(node->rightNode);
+    if(index < leftNode->end)
+      node = leftNode;
+    else {
+      const SetNodeData* rightNode = nodeFromIndex(node->rightNode);
+      node = rightNode;
+    }
   }
   
   return false;
@@ -1109,31 +1080,38 @@ Set BasicSetRepository::createSetFromRanges(const std::vector<Index>& indices) {
 }
 
 Set BasicSetRepository::createSet(Index i) {
-  std::vector<Index> ranges;
-  ranges.push_back(i);
-  ranges.push_back(i+1);
-  return createSetFromRanges(ranges);
+    SetNodeData data;
+    data.start = i;
+    data.end = i+1;
+    data.updateHash(0, 0);
+    
+    return Set(d->dataRepository.index( SetNodeDataRequest(&data, d->dataRepository) ), this);
 }
 
-Set BasicSetRepository::createSet(const std::set<Index>& _indices) {
+Set BasicSetRepository::createSet(const std::set<Index>& indices) {
+  QMutexLocker lock(d->mutex);
   
-  std::set<Index> indices(_indices); //Create a copy to prevent strange problems
+  if(indices.empty())
+      return Set();
   
-  std::vector<Index> ranges;
+  std::vector<Index> shortRanges(indices.size()*2);
 
+  uint size = 0;
   for( std::set<Index>::const_iterator it = indices.begin(); it != indices.end(); ++it )
   {
-    if(ranges.empty() || ranges.back() != *it)
-    {
-      //Create new range
-      ranges.push_back(*it);
-      ranges.push_back(*it);
-    }
-    
-    ++ranges.back();
+    shortRanges[size] = *it;
+    ++size;
+    shortRanges[size] = *it+1;
+    ++size;
   }
+  
+  uint* ranges = &shortRanges[0];
+  
+  QVector<SplitTreeNode> splitTree;
+  
+  splitTreeForRanges(splitTree, &ranges, &size, 1);
 
-  return createSetFromRanges(ranges);
+  return Set(d->dataRepository.index( SetNodeDataRequest(&splitTree[0], d->dataRepository) ), this);
 }
 
 BasicSetRepository::BasicSetRepository(QString name, bool doLocking) : d(new Private(name, doLocking)) {
@@ -1167,6 +1145,16 @@ BasicSetRepository::~BasicSetRepository() {
   delete d;
 }
 
+void BasicSetRepository::itemRemovedFromSets(uint /*index*/) {
+}
+#undef nodeFromIndex
+const SetNodeData* BasicSetRepository::nodeFromIndex(uint index) const {
+    if(index)
+        return d->dataRepository.itemFromIndex(index);
+    else
+        return 0;
+}
+
 ////////////Set convenience functions//////////////////
 
 bool Set::contains(Index index) const
@@ -1330,8 +1318,5 @@ void Set::staticUnref() {
     }
 }
 
-void BasicSetRepository::itemRemovedFromSets(uint /*index*/) {
-}
-
 }
 
