@@ -56,6 +56,7 @@
 #include <language/duchain/navigation/abstractdeclarationnavigationcontext.h>
 #include <interfaces/contextmenuextension.h>
 #include <interfaces/iplugincontroller.h>
+#include <ktexteditor/codecompletioninterface.h>
 
 const int maxHistoryLength = 30;
 
@@ -73,7 +74,7 @@ QToolButton* ContextController::browseButton() const {
     return m_browseButton;
 }
 
-ContextController::ContextController(ContextBrowserView* view) : m_nextHistoryIndex(0), m_view(view) {
+ContextController::ContextController(ContextBrowserView* view) : QObject(view), m_nextHistoryIndex(0), m_view(view) {
     m_browseManager = new BrowseManager(this);
     
     connect(ICore::self()->documentController(), SIGNAL(documentJumpPerformed(KDevelop::IDocument*, KTextEditor::Cursor, KDevelop::IDocument*, KTextEditor::Cursor)), this, SLOT(documentJumpPerformed(KDevelop::IDocument*, KTextEditor::Cursor, KDevelop::IDocument*, KTextEditor::Cursor)));
@@ -117,6 +118,67 @@ DUContext* getContextAt(KUrl url, KTextEditor::Cursor cursor) {
     return contextAt(SimpleCursor(cursor), topContext);
 }
 
+static bool useNavigationFromView(QObject* viewObject) {
+    KTextEditor::View* view = qobject_cast<KTextEditor::View*>(viewObject);
+    if(!view) {
+        kWarning() << "sender is not a view";
+        return false;
+    }
+    KTextEditor::CodeCompletionInterface* iface = dynamic_cast<KTextEditor::CodeCompletionInterface*>(view);
+    if(!iface || iface->isCompletionActive())
+        return false;
+    
+    return true;
+}
+
+void ContextBrowserView::navigateAccept() {
+    if(!useNavigationFromView(sender()))
+        return;
+    
+    if(AbstractNavigationWidget* widget = dynamic_cast<AbstractNavigationWidget*>(m_navigationWidget.data()))
+        widget->accept();
+}
+
+void ContextBrowserView::navigateBack() {
+    if(!useNavigationFromView(sender()))
+        return;
+    
+    if(AbstractNavigationWidget* widget = dynamic_cast<AbstractNavigationWidget*>(m_navigationWidget.data()))
+        widget->back();
+}
+
+void ContextBrowserView::navigateDown() {
+    if(!useNavigationFromView(sender()))
+        return;
+    
+    if(AbstractNavigationWidget* widget = dynamic_cast<AbstractNavigationWidget*>(m_navigationWidget.data()))
+        widget->down();
+}
+
+void ContextBrowserView::navigateLeft() {
+    if(!useNavigationFromView(sender()))
+        return;
+    
+    if(AbstractNavigationWidget* widget = dynamic_cast<AbstractNavigationWidget*>(m_navigationWidget.data()))
+        widget->previous();
+}
+
+void ContextBrowserView::navigateRight() {
+    if(!useNavigationFromView(sender()))
+        return;
+    
+    if(AbstractNavigationWidget* widget = dynamic_cast<AbstractNavigationWidget*>(m_navigationWidget.data()))
+        widget->next();
+}
+
+void ContextBrowserView::navigateUp() {
+    if(!useNavigationFromView(sender()))
+        return;
+    
+    if(AbstractNavigationWidget* widget = dynamic_cast<AbstractNavigationWidget*>(m_navigationWidget.data()))
+        widget->up();
+}
+
 void ContextController::documentJumpPerformed( KDevelop::IDocument* newDocument, KTextEditor::Cursor newCursor, KDevelop::IDocument* previousDocument, KTextEditor::Cursor previousCursor) {
     if(newCursor.isValid() && previousCursor.isValid()) {
         
@@ -445,7 +507,7 @@ void ContextBrowserView::resetWidget()
 void ContextBrowserView::declarationMenu() {
     DUChainReadLocker lock(DUChain::lock());
     
-    AbstractNavigationWidget* navigationWidget = dynamic_cast<AbstractNavigationWidget*>(m_navigationWidget);
+    AbstractNavigationWidget* navigationWidget = dynamic_cast<AbstractNavigationWidget*>(m_navigationWidget.data());
     if(navigationWidget) {
         AbstractDeclarationNavigationContext* navigationContext = dynamic_cast<AbstractDeclarationNavigationContext*>(navigationWidget->context().data());
         if(navigationContext && navigationContext->declaration().data()) {
@@ -470,6 +532,9 @@ ContextBrowserView::ContextBrowserView( ContextBrowserPlugin* plugin ) : m_plugi
     m_declarationCtrl = new DeclarationController();
     m_contextCtrl = new ContextController(this);
     
+    connect(m_contextCtrl->browseManager(), SIGNAL(startDelayedBrowsing(KTextEditor::View*)), this, SIGNAL(startDelayedBrowsing(KTextEditor::View*)));
+    connect(m_contextCtrl->browseManager(), SIGNAL(stopDelayedBrowsing()), this, SIGNAL(stopDelayedBrowsing()));
+    
     m_allowLockedUpdate = false;
     
     m_buttons = new QHBoxLayout;
@@ -511,7 +576,7 @@ ContextBrowserView::~ContextBrowserView() {
 void ContextBrowserView::focusInEvent(QFocusEvent* event) {
     //Indicate that we have focus
     kDebug() << "got focus";
-    parentWidget()->setBackgroundRole(QPalette::ToolTipBase);
+//     parentWidget()->setBackgroundRole(QPalette::ToolTipBase);
 /*    m_layout->removeItem(m_buttons);*/
     
     return QWidget::focusInEvent(event);
@@ -519,7 +584,7 @@ void ContextBrowserView::focusInEvent(QFocusEvent* event) {
 
 void ContextBrowserView::focusOutEvent(QFocusEvent* event) {
     kDebug() << "lost focus";
-    parentWidget()->setBackgroundRole(QPalette::Background);
+//     parentWidget()->setBackgroundRole(QPalette::Background);
 /*    m_layout->insertLayout(0, m_buttons);
     for(int a = 0; a < m_buttons->count(); ++a) {
         QWidgetItem* item = dynamic_cast<QWidgetItem*>(m_buttons->itemAt(a));
@@ -531,13 +596,7 @@ bool ContextBrowserView::event(QEvent* event) {
     QKeyEvent* keyEvent = dynamic_cast<QKeyEvent*>(event);
     
     if(hasFocus() && keyEvent) {
-        if(m_shiftDetector.checkKeyEvent(keyEvent) || (event->type() == QEvent::KeyPress && keyEvent->key() == Qt::Key_Escape)) {
-            //Switch the focus back to the editor view
-            kDebug() << "switching back to" << m_contextCtrl->focusBackWidget();
-            if(m_contextCtrl->focusBackWidget())
-                m_contextCtrl->focusBackWidget()->setFocus();
-        }
-        AbstractNavigationWidget* navigationWidget = dynamic_cast<AbstractNavigationWidget*>(m_navigationWidget);
+        AbstractNavigationWidget* navigationWidget = dynamic_cast<AbstractNavigationWidget*>(m_navigationWidget.data());
         if(navigationWidget && event->type() == QEvent::KeyPress) {
             int key = keyEvent->key();
             if(key == Qt::Key_Left)
@@ -654,6 +713,10 @@ void ContextBrowserView::allowLockedUpdate() {
     m_allowLockedUpdate = true;
 }
 
+void ContextBrowserView::setNavigationWidget(QWidget* widget) {
+    updateMainWidget(widget);
+}
+
 void ContextBrowserView::setContext(KDevelop::DUContext* context) {
     if(!context)
         return;
@@ -682,4 +745,9 @@ void ContextBrowserView::setSpecialNavigationWidget(QWidget* widget) {
     }
 }
 
+BrowseManager* ContextController::browseManager() const {
+    return m_browseManager;
+}
+
+
 #include "contextbrowserview.moc"
