@@ -56,15 +56,16 @@ public:
     KDevelop::ContextMenuExtension contextMenuExtension( KDevelop::Context* );
 
     ProjectBuildSetModel* buildSet();
-
+public Q_SLOTS:
+    void buildProjectItems();
+    void installProjectItems();
+    void cleanProjectItems();
+ 
 protected Q_SLOTS:
     void closeProjects();
     void buildItemsFromContextMenu();
     void installItemsFromContextMenu();
     void cleanItemsFromContextMenu();
-    void buildProjectItems();
-    void installProjectItems();
-    void cleanProjectItems();
     void configureProjectItems();
     void pruneProjectItems();
     void buildAllProjects();
@@ -77,11 +78,6 @@ protected Q_SLOTS:
     void removeFolderFromContextMenu();
 
 private:
-    void executeBuild( KDevelop::ProjectBaseItem* );
-    void executeClean( KDevelop::ProjectBaseItem* );
-    void executeInstall( KDevelop::ProjectBaseItem* );
-    void executePrune( KDevelop::IProject* );
-    void executeConfigure( KDevelop::IProject* );
     static KDevelop::IProjectBuilder* getProjectBuilder( KDevelop::ProjectBaseItem* item );
     QList<KDevelop::ProjectBaseItem*> recurseAndFetchCheckedItems( KDevelop::ProjectBaseItem* item );
     class ProjectManagerViewPluginPrivate* const d;
