@@ -49,7 +49,6 @@ Boston, MA 02110-1301, USA.
 #include <interfaces/iplugin.h>
 #include <interfaces/isession.h>
 
-#include "profileengine.h"
 #include "mainwindow.h"
 #include "core.h"
 #include "shellextension.h"
@@ -57,6 +56,22 @@ Boston, MA 02110-1301, USA.
 namespace KDevelop
 {
 
+static const QString pluginControllerGrp("Plugins");
+
+bool isGlobalPlugin( const KPluginInfo& info )
+{
+    return info.property( "X-KDevelop-Category" ).toString() == "Global";
+}
+
+bool hasMandatoryProperties( const KPluginInfo& info )
+{
+    QVariant mode = info.property( "X-KDevelop-Mode" );
+    QVariant version = info.property( "X-KDevelop-Version" );
+
+    return mode.isValid() && mode.canConvert( QVariant::String )
+           && version.isValid() && version.canConvert( QVariant::String );
+}
+
 class PluginControllerPrivate
 {
 public:
@@ -78,8 +93,6 @@ public:
     };
     CleanupMode cleanupMode;
 
-    QString profile;
-    ProfileEngine engine;
     Core *core;
     QExtensionManager* m_manager;
 };
@@ -89,12 +102,18 @@ PluginController::PluginController(Core *core)
 {
     setObjectName("PluginController");
     d->core = core;
-    d->profile = ShellExtension::getInstance() ->defaultProfile();
-    kDebug() << "Loading plugins which match:" << QString( "[X-KDevelop-Version] == %1" ).arg(KDEVELOP_PLUGIN_VERSION);
+    kDebug() << "Fetching plugin info which matches:" << QString( "[X-KDevelop-Version] == %1" ).arg(KDEVELOP_PLUGIN_VERSION);
     d->plugins = KPluginInfo::fromServices( KServiceTypeTrader::self()->query( QLatin1String( "KDevelop/Plugin" ),
         QString( "[X-KDevelop-Version] == %1" ).arg(KDEVELOP_PLUGIN_VERSION) ) );
+    foreach( KPluginInfo p, d->plugins )
+    {
+        if( p.pluginName().contains("nongui") )
+                kDebug() << "FOUND test plugin" << p.pluginName();
+    }
     d->cleanupMode = PluginControllerPrivate::Running;
     d->m_manager = new QExtensionManager();
+    // Register the KDevelop::IPlugin* metatype so we can properly unload it
+    qRegisterMetaType<KDevelop::IPlugin*>( "KDevelop::IPlugin*" );
 }
 
 PluginController::~PluginController()
@@ -107,16 +126,6 @@ PluginController::~PluginController()
     delete d;
 }
 
-QString PluginController::currentProfile() const
-{
-    return d->profile;
-}
-
-ProfileEngine& PluginController::engine() const
-{
-    return d->engine;
-}
-
 KPluginInfo PluginController::pluginInfo( const IPlugin* plugin ) const
 {
     for ( PluginControllerPrivate::InfoToPluginMap::ConstIterator it = d->loadedPlugins.begin();
@@ -153,32 +162,43 @@ IPlugin* PluginController::loadPlugin( const QString& pluginName )
     return loadPluginInternal( pluginName );
 }
 
-void PluginController::loadPlugins( PluginType type )
+void PluginController::initialize()
 {
-    KPluginInfo::List offers = d->engine.offers( d->profile, type );
-    KConfigGroup grp = Core::self()->activeSession()->config()->group( "Plugins" );
-    QStringList sessionPlugins = grp.readEntry( "Plugins To Load", QStringList() );
-    foreach( const KPluginInfo& pi, offers )
-    {
-        if( sessionPlugins.isEmpty() || sessionPlugins.contains( pi.pluginName() ) )
-        {
-            loadPluginInternal( pi.pluginName() );
-        }
-    }
-}
+    KConfigGroup grp = Core::self()->activeSession()->config()->group( pluginControllerGrp );
 
-void PluginController::unloadPlugins( PluginType type )
-{
-    //TODO see if this can be optimized so it's not something like O(n^2)
-    KPluginInfo::List offers = d->engine.offers( d->profile, type );
-    foreach( const KPluginInfo& pi, offers )
+    QMap<QString, QString> entries = grp.entryMap();
+
+    QMap<QString, bool> pluginMap;
+
+    QMap<QString, QString>::Iterator it;
+    for ( it = entries.begin(); it != entries.end(); ++it )
     {
-        foreach ( const KPluginInfo& lpi, d->loadedPlugins.keys() )
+        QString key = it.key();
+        if ( key.endsWith( QLatin1String( "Enabled" ) ) )
+            pluginMap.insert( key.left(key.length() - 7), grp.readEntry(key,false) );
+    }
+
+    // If pluginMap is empty here (i.e. nothing in the session) use the list available
+    // from the shellextension
+    foreach( const QString& s, ShellExtension::getInstance()->defaultPlugins() )
+    {
+        pluginMap.insert( s, true );
+    }
+
+    foreach( const KPluginInfo& pi, d->plugins )
+    {
+        if( isGlobalPlugin( pi ) )
         {
-            if ( pi.pluginName() == lpi.pluginName() )
-                unloadPlugin( pi.pluginName() );
+            QMap<QString, bool>::const_iterator it = pluginMap.constFind( pi.pluginName() );
+            if( pluginMap.isEmpty() || ( it != pluginMap.constEnd() && it.value() ) && loadPluginInternal( pi.pluginName() ) != 0 )
+            {
+                loadPluginInternal( pi.pluginName() );
+                grp.writeEntry( pi.pluginName()+"Enabled", true );
+            }
         }
     }
+    // Synchronize so we're writing out to the file.
+    grp.sync();
 }
 
 QList<IPlugin *> PluginController::loadedPlugins() const
@@ -220,46 +240,6 @@ void PluginController::unloadPlugin(IPlugin* plugin, PluginDeletion deletion)
         delete plugin;
 }
 
-KUrl::List PluginController::profileResources( const QString &nameFilter )
-{
-    return d->engine.resources( currentProfile(), nameFilter );
-}
-
-KUrl::List PluginController::profileResourcesRecursive( const QString &nameFilter )
-{
-    return d->engine.resourcesRecursive( currentProfile(), nameFilter );
-}
-
-QString PluginController::changeProfile( const QString &newProfile )
-{
-	Q_UNUSED( newProfile );
-    /* FIXME disabled for now
-    QStringList unload;
-    KService::List coreLoad;
-    KService::List globalLoad;
-    d->engine.diffProfiles( ProfileEngine::Core,
-                           currentProfile(),
-                           newProfile,
-                           unload,
-                           coreLoad );
-    d->engine.diffProfiles( ProfileEngine::Global,
-                           currentProfile(),
-                           newProfile,
-                           unload,
-                           globalLoad );
-
-    QString oldProfile = d->profile;
-    d->profile = newProfile;
-
-    unloadPlugins( unload );
-    loadPlugins( coreLoad );
-    loadPlugins( globalLoad );
-
-    return oldProfile;
-    */
-    return QString();
-}
-
 KPluginInfo PluginController::infoForPluginId( const QString &pluginId ) const
 {
     QList<KPluginInfo>::ConstIterator it;
@@ -281,9 +261,33 @@ IPlugin *PluginController::loadPluginInternal( const QString &pluginId )
         return 0L;
     }
 
+    // Do not load KDevKrossManager directly, its indirectly loaded when loading
+    // any plugins written in a supported scripting language
+    // The kross-manager plugin needs the name+interfaces of the script-plugin
+    // as argument
+    // At a later point in time, we should try to move the plugin's code directly
+    // into shell
+    if( info.pluginName() == "KDevKrossManager" )
+    {
+        kDebug() << "tried to load KDevKrossManager, ignoring";
+        return 0;
+    }
+
     if ( d->loadedPlugins.contains( info ) )
         return d->loadedPlugins[ info ];
 
+    if( !hasMandatoryProperties( info ) ) {
+        kWarning() << "Unable to load plugin named " << pluginId << "! Doesn't have all mandatory properties set";
+        return 0;
+    }
+
+    if( info.property("X-KDevelop-Mode") == "GUI"
+        && Core::self()->setupFlags() == Core::NoUi )
+    {
+        kWarning() << "Unable to load plugin named" << pluginId << ". Running in No-Ui mode, but the plugin says it needs a GUI";
+        return 0;
+    }
+
     kDebug() << "Attempting to load '" << pluginId << "'";
     emit loadingPlugin( info.pluginName() );
     QString str_error;
@@ -361,7 +365,7 @@ bool PluginController::checkForDependencies( const KPluginInfo& info, QStringLis
     if( prop.canConvert<QStringList>() )
     {
         QStringList deps = prop.toStringList();
-        foreach( QString iface, deps )
+        foreach( const QString &iface, deps )
         {
             KPluginInfo::List l = queryPlugins( QString("'%1' in [X-KDevelop-Interfaces]").arg(iface) );
             if( l.isEmpty() )
@@ -380,7 +384,7 @@ void PluginController::loadOptionalDependencies( const KPluginInfo& info )
     if( prop.canConvert<QStringList>() )
     {
         QStringList deps = prop.toStringList();
-        foreach( QString iface, deps )
+        foreach( const QString &iface, deps )
         {
             KPluginInfo info = queryPlugins( QString("'%1' in [X-KDevelop-Interfaces]").arg(iface) ).first();
             if( !loadPluginInternal( info.pluginName() ) )
@@ -397,7 +401,7 @@ void PluginController::loadDependencies( const KPluginInfo& info )
     if( prop.canConvert<QStringList>() )
     {
         QStringList deps = prop.toStringList();
-        foreach( QString iface, deps )
+        foreach( const QString &iface, deps )
         {
             KPluginInfo info = queryPlugins( QString("'%1' in [X-KDevelop-Interfaces]").arg(iface) ).first();
             loadPluginInternal( info.pluginName() );
@@ -422,7 +426,7 @@ IPlugin *PluginController::pluginForExtension(const QString &extension, const QS
 
     if( infos.isEmpty() )
         return 0;
-    if( d->plugins.contains( infos.first() ) )
+    if( d->loadedPlugins.contains( infos.first() ) )
         return d->loadedPlugins[ infos.first() ];
     else
         return loadPluginInternal( infos.first().pluginName() );
@@ -436,7 +440,7 @@ QList<IPlugin*> PluginController::allPluginsForExtension(const QString &extensio
     foreach (const KPluginInfo &info, infos)
     {
         IPlugin* plugin;
-        if( d->plugins.contains( info ) )
+        if( d->loadedPlugins.contains( info ) )
             plugin = d->loadedPlugins[ info ];
         else
             plugin = loadPluginInternal( info.pluginName() );
@@ -493,6 +497,52 @@ QStringList PluginController::projectPlugins()
     return names;
 }
 
+void PluginController::loadProjectPlugins()
+{
+    Q_FOREACH( const QString& name, projectPlugins() )
+    {
+        loadPluginInternal( name );
+    }
+}
+
+void PluginController::unloadProjectPlugins()
+{
+    Q_FOREACH( const QString& name, projectPlugins() )
+    {
+        unloadPlugin( name );
+    }
+}
+
+QList<KPluginInfo> PluginController::allPluginInfos() const
+{
+    return d->plugins;
+}
+
+void PluginController::updateLoadedPlugins()
+{
+}
+
+void PluginController::resetToDefaults()
+{
+    KSharedConfig::Ptr cfg = Core::self()->activeSession()->config();
+    cfg->deleteGroup( pluginControllerGrp );
+    cfg->sync();
+    KConfigGroup grp = cfg->group( pluginControllerGrp );
+    QStringList plugins = ShellExtension::getInstance()->defaultPlugins();
+    if( plugins.isEmpty() ) 
+    {
+        foreach( const KPluginInfo& info, d->plugins )
+        {
+            plugins << info.pluginName();
+        }
+    }
+    foreach( const QString& s, plugins )
+    {
+        grp.writeEntry( s+"Enabled", true );
+    }
+    grp.sync();
+}
+
 }
 #include "plugincontroller.moc"
 
